//******************** (C) D.Buergin *******************************************
//* File Name          : lcd.c
//* Author             : D. Buergin
//* Version            : V1.0
//* Date               : 19-02-2009
//* Description        : LCD Routines for Cortex M3 CPU
//******************************************************************************

//* Includes ------------------------------------------------------------------*

#include "lcd.h"

//* Private typedef -----------------------------------------------------------*
//* Private define ------------------------------------------------------------*
//* Private macro -------------------------------------------------------------*
//* Private variables ---------------------------------------------------------*
//* Private function prototypes -----------------------------------------------*
//* Private functions ---------------------------------------------------------*

//****************************
//* Strobe 4-Bit Data to LCD *
//****************************

void lcd_out_data4(unsigned char val)
{   
  if((val&0x01)==0x01)	// Bit[0] 
  {
    LCD_D4_HI();  
  }  
  else
  {
    LCD_D4_LO();
  }
  
  if((val&0x02)==0x02)  // Bit[1] 
  {
    LCD_D5_HI();  
  }  
  else
  {
    LCD_D5_LO();
  }
  
  if((val&0x04)==0x04)  // Bit[2] 
  {
    LCD_D6_HI();  
  }  
  else
  {
    LCD_D6_LO();
  } 
  
  if((val&0x08)==0x08)  // Bit[3]
  {
    LCD_D7_HI();  
  }  
  else
  {
    LCD_D7_LO();
  } 
  
}

//****************************
//* Write Data 1 Byte to LCD *
//****************************

void lcd_write_byte(unsigned char val)
{  
  lcd_out_data4((val>>4)&0x0F);							// Strobe 4-Bit High-Nibble to LCD
  enable_lcd();											// Enable Pulse
  
  lcd_out_data4(val&0x0F);				  				// Strobe 4-Bit Low-Nibble to LCD
  enable_lcd();											// Enable Pulse  

  while(busy_lcd());      								// Wait LCD Execute Complete  
}

//****************************
//* Write Instruction to LCD *
//****************************

void lcd_write_control(unsigned char val)
{ 
  LCD_RS_LO();											// RS = 0 = Instruction Select
  lcd_write_byte(val);									// Strobe Command Byte	    
}

//****************************
//* Write Data(ASCII) to LCD *
//****************************

void lcd_write_ascii(unsigned char c)
{  
  LCD_RS_HI();											// RS = 1 = Data Select
  lcd_write_byte(c);		   							// Strobe 1 Byte to LCD    
}

//*******************************
//* Initial 4-Bit LCD Interface *
//*******************************

void lcd_init(void)
{
  /* Configure IO connected to LCD16X2 */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_EN, ENABLE);
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = LCD_EN_PIN;
  GPIO_Init(LCD_EN_PORT, &GPIO_InitStructure);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_RW, ENABLE);
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = LCD_RW_PIN;
  GPIO_Init(LCD_RW_PORT, &GPIO_InitStructure);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_RS, ENABLE);
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = LCD_RS_PIN;
  GPIO_Init(LCD_RS_PORT, &GPIO_InitStructure);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_DATA, ENABLE);
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = LCD_D4_PIN |
                                LCD_D5_PIN |
								LCD_D6_PIN |
                                LCD_D7_PIN;
  GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);

  LCD_D4_HI();
  LCD_D5_HI();
  LCD_D6_LO();
  LCD_D7_LO();
  DelayuS(15000);			// Power-On Delay (15 mS)
  
  LCD_D4_HI();
  LCD_D5_HI();
  LCD_D6_LO();
  LCD_D7_LO();
  enable_lcd();				// Enable Pulse
  DelayuS(4100);			// Delay 4.1mS
  
  LCD_D4_HI();
  LCD_D5_HI();
  LCD_D6_LO();
  LCD_D7_LO();
  enable_lcd();				// Enable Pulse
  DelayuS(100);				// delay 100uS
   
  LCD_D4_HI();
  LCD_D5_HI();
  LCD_D6_LO();
  LCD_D7_LO();
  enable_lcd();				// Enable Pulse
  while(busy_lcd());      	// Wait LCD Execute Complete
   
  LCD_D4_LO();
  LCD_D5_HI();
  LCD_D6_LO();
  LCD_D7_LO();
  enable_lcd();				// Enable Pulse
  while(busy_lcd());      	// Wait LCD Execute Complete
       
  lcd_write_control(0x28); 	// Function Set (DL=0 4-Bit,N=1 2 Line,F=0 5X7)
  lcd_write_control(0x0C); 	// Display on/off Control (Entry Display,Cursor off,Cursor not Blink)
  lcd_write_control(0x06);  // Entry Mode Set (I/D=1 Increment,S=0 Cursor Shift)
  lcd_write_control(0x01); 	// Clear Display  (Clear Display,Set DD RAM Address=0) 
  Delay(15);  			// Wait Command Ready  
}

//***************************
//* Set LCD Position Cursor *
//* x = column              *
//* y = row                 *
//***************************

void lcd_gotoxy(unsigned char x, unsigned char y)
{
#if LCD_LINES==1
    lcd_write_control((1<<LCD_DDRAM)+LCD_START_LINE1+x);
#endif
#if LCD_LINES==2
    if ( y==0 )
        lcd_write_control((1<<LCD_DDRAM)+LCD_START_LINE1+x);
    else
        lcd_write_control((1<<LCD_DDRAM)+LCD_START_LINE2+x);
#endif
#if LCD_LINES==4
    if ( y==0 )
        lcd_write_control((1<<LCD_DDRAM)+LCD_START_LINE1+x);
    else if ( y==1)
        lcd_write_control((1<<LCD_DDRAM)+LCD_START_LINE2+x);
    else if ( y==2)
        lcd_write_control((1<<LCD_DDRAM)+LCD_START_LINE3+x);
    else /* y==3 */
        lcd_write_control((1<<LCD_DDRAM)+LCD_START_LINE4+x);
#endif

}

/************************************/
/* Print Display Data(ASCII) to LCD */
/************************************/

void lcd_print(char* str)
{
  int i;

  for (i=0;i<20 && str[i]!=0;i++)  						// 16 Character Print
  {
    lcd_write_ascii(str[i]);							// Print Byte to LCD
  }
}

/******************/
/* Wait LCD Ready */
/******************/

char busy_lcd(void)
{  
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;		// Config D7 = Read
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_InitStructure.GPIO_Pin = LCD_D7_PIN;
  GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);

  LCD_RS_LO();		 									// Instruction Select
  LCD_RW_HI(); 											// Read Direction
  LCD_EN_HI();											// Start Read Busy

  DelayuS(100);	  										// Delay Before Read
  if (GPIO_ReadInputDataBit(LCD_DATA_PORT, LCD_D7_PIN) == Bit_SET)
  {
    LCD_EN_LO();  										// Disable Read
  	LCD_RW_LO();										// Default = Write Direction
  	
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;  	// Config D7 = Write
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Pin = LCD_D7_PIN;
    GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);
    return 1;											// LCD Busy Status
  }
  else
  {
    LCD_EN_LO();  										// Disable Read
  	LCD_RW_LO();										// Default = Write Direction
  	
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	// Config D7 = Write
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Pin = LCD_D7_PIN;
    GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);
    return 0;											// LCD Ready Status
  }  
}


/***********************/
/* Enable Pulse to LCD */
/***********************/
void enable_lcd(void)	 								// Enable Pulse
{  
  LCD_EN_HI();  										// Enable ON
  DelayuS(50);  
  LCD_EN_LO();  										// Enable OFF 
}
