/**
  ******************************************************************************
  * @file SysTick/stm32f10x_it.c
  * @author  MCD Application Team
  * @version  V3.0.0
  * @date  04/06/2009
  * @brief  Main Interrupt Service Routines.
  *         This file provides template for all exceptions handler and
  *         peripherals interrupt service routine.
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2009 STMicroelectronics</center></h2>
  */

/* mthomas: not used, kept for reference, weakly defined handlers directly
 * overwritten (i.e. SysTick_Handler in main.c)
 */

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x_it.h"
#include "platform_config.h"
extern void TimingDelay_Decrement(void);

/** @addtogroup StdPeriph_Examples
  * @{
  */

/** @addtogroup SysTick
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M3 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief  This function handles NMI exception.
  * @param  None
  * @retval : None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval : None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval : None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval : None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval : None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval : None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval : None
  */
void DebugMon_Handler(void)
{
}

/**
  * @brief  This function handles PendSV_Handler exception.
  * @param  None
  * @retval : None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval : None
  */
void SysTick_Handler(void)
{

	static uint8_t counter;

	//=====================================================================================
	// Routine stammt von Peter Danneger und enthaelt die
	// Entprellroutinen fuer die Taster
	//=====================================================================================

	// static uint8_t ct0, ct1, rpt;
	// uint8_t i;

	// i = key_state ^ ~ButtonPIN;     // key changed ?
	// ct0 = ~( ct0 & i );         // reset or count ct0
	// ct1 = ct0 ^ (ct1 & i);      // reset or count ct1
	// i &= ct0 & ct1;         // count until roll over ?
	// key_state ^= i;         // then toggle debounced state
	// key_press |= key_state & i;     // 0->1: key press detect

	// if( (key_state & REPEAT_MASK) == 0 )    // check repeat function
	// {
// 		rpt = REPEAT_START;     // start delay
// 	}

// 	if( --rpt == 0 )
// 	{
// 		rpt = REPEAT_NEXT;          // repeat delay
// 		key_rpt |= key_state & REPEAT_MASK;
// 	}

    //=====================================================================================
	// Normale SysTick Routine fuer Delay()
	//=====================================================================================

	TimingDelay_Decrement();

	counter++;
	if (counter > 250)
	{
  		GPIO_WriteBit(GPIO_LED, GPIO_Pin_13, (BitAction)((1-GPIO_ReadOutputDataBit(GPIO_LED, GPIO_Pin_13))));
		counter = 0;
	}
}

/*******************************************************************************
 * * Function Name  : TIM1_UP_IRQHandler
 * * Description    : This function handles TIM1 overflow and update interrupt
 * *                  request.
 * * Input          : None
 * * Output         : None
 * * Return         : None
 * *******************************************************************************/

void TIM1_UP_IRQHandler(void)
{
  static int32_t enc_last = 0x01;
  int8_t i = 0;
  int8_t richtung = 0;

  // Den Timer wieder aktivieren

  TIM_ClearITPendingBit(TIM1, TIM_IT_Update);

  // ==============================================================================
  // Den Encoder abfragen
  // Routine von Peter Dannegger aus www.mikrocontroller.net
  // ==============================================================================

  if ((GPIO_ENC->IDR & ENC1_UA1) != (uint32_t)Bit_RESET)
    i = 1;

  if ((GPIO_ENC->IDR & ENC1_UA2) != (uint32_t)Bit_RESET)
    i ^= 3;                    // convert gray to binary

  i -= enc_last;                     // difference new - last

  if( i & 1 )
  {                          // bit 0 = value (1)
    enc_last += i;             // store new as next last
    enc_delta += (i & 2) - 1;  // bit 1 = direction (+/-)

    richtung = (i & 2) - 1;
  }
}

/******************* (C) COPYRIGHT 2009 STMicroelectronics *****END OF FILE****/
